#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QTreeWidgetItem>
#include <QToolButton>

#include <QGraphicsScene>
#include <QAbstractGraphicsShapeItem>
#include <QGraphicsRectItem>
#include <QGraphicsEllipseItem>

namespace Ui {
class MainWindow;
}

class ParamList : public QMap <QString, QString>
{
   public:
      int number (QString key, int default_value = 0);
};

class ColorButton : public QToolButton
{
private :
    QString name;
    QColor color;
public :
    ColorButton (QString s);
protected:
    void mousePressEvent(QMouseEvent *event);
};

class Common : private QObject
{
Q_OBJECT
public :
    ParamList params;
};

class Rectangle : public QGraphicsRectItem, public Common
{
protected:
    void dragEnterEvent (QGraphicsSceneDragDropEvent *event);
    void dropEvent (QGraphicsSceneDragDropEvent *event);
public:
    Rectangle ();
};

class Ellipse : public QGraphicsEllipseItem, public Common
{
protected:
    void dragEnterEvent (QGraphicsSceneDragDropEvent *event);
    void dropEvent (QGraphicsSceneDragDropEvent *event);
public:
    Ellipse ();
};


class TreeNode : public QTreeWidgetItem
{
public:
    Common * common;
    QAbstractGraphicsShapeItem * shape;
    TreeNode () : common (NULL), shape (NULL) { }
};

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

    void readFile(QString fileName);

    Rectangle * createRectangle (TreeNode * node,
                                 QGraphicsItem * target,
                                 ParamList param);

    Ellipse * createEllipse (TreeNode * node,
                             QGraphicsItem * target,
                             ParamList param);

    void modifyShape (QGraphicsItem * target,
                      QAbstractGraphicsShapeItem * shape,
                      ParamList param);

private slots:
    void on_treeWidget_itemClicked(QTreeWidgetItem *item, int column);

private:
    Ui::MainWindow * ui;
    QGraphicsScene * scene;
};

#endif // MAINWINDOW_H
